PrimeUIHandler = class(Turbine.UI.Control)
function PrimeUIHandler:Constructor()
    Turbine.UI.Control.Constructor(self)

    self.tools = {}

    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()

    self.ClassLut = {
        [Turbine.Gameplay.Class.Burglar] = "Burglar",
        [Turbine.Gameplay.Class.Captain] = "Captain",
        [Turbine.Gameplay.Class.Champion] = "Champion",
        [Turbine.Gameplay.Class.Guardian] = "Guardian",
        [Turbine.Gameplay.Class.Hunter] = "Hunter",
        [Turbine.Gameplay.Class.LoreMaster] = "LoreMaster",
        [Turbine.Gameplay.Class.Minstrel] = "Minstrel",
        [Turbine.Gameplay.Class.RuneKeeper] = "RuneKeeper",
        [Turbine.Gameplay.Class.Warden] = "Warden",
        [Turbine.Gameplay.Class.Beorning] = "Beorning",
        [Turbine.Gameplay.Class.Brawler] = "Brawler",
        [Turbine.Gameplay.Class.BlackArrow] = "BlackArrow",
        [Turbine.Gameplay.Class.Defiler] = "Defiler",
        [Turbine.Gameplay.Class.Reaver] = "Reaver",
        [Turbine.Gameplay.Class.Stalker] = "Stalker",
        [Turbine.Gameplay.Class.WarLeader] = "WarLeader",
        [Turbine.Gameplay.Class.Weaver] = "Weaver",
    } 

    self.DungeonNameTable = InstanceController:GetDungeonNameTable()
end

function PrimeUIHandler:Initialize()
    self.UIData = LoadTable(Turbine.DataScope.Account, "PrimeUI")

    if self.UIData == nil then
        self.UIData = {}
    end

    self:Setup()
end

function PrimeUIHandler:Setup()
    for key, value in pairs(self.UIData) do
        if self:Permission(value) then
            --self.tools[value.Name] = PrimeUIObjects.PrimeUI(value.PrimeUI, value.Name, value.X, value.Y, value.AdvancedMode, value.OOCLoad, value.ICLoad, value.Anchor_Advanced_X, value.Anchor_Advanced_Y)
            -- [[
            local success, result = pcall(PrimeUIObjects.PrimeUI, value.PrimeUI, value.Name, value.X, value.Y, value.AdvancedMode, value.OOCLoad, value.ICLoad, value.Anchor_Advanced_X, value.Anchor_Advanced_Y)
            if success then
                self.tools[value.Name] = result
            else
                ChatOutput(string.format("Failed to properly load PrimeUI \"%s\".", value.Name))
            end--]]
        end
    end
end

function PrimeUIHandler:Permission(data)   
    if data.NeverLoad then return false end
    return not(data.NeverLoad) and
        (data.OnlyAllDelvings == nil or data.OnlyAllDelvings and Settings.DelvingMissionActive or data.OnlyAllDelvings == false) and
        (data.Delving == nil or data.Delving == "All" or data.Delving == Settings.MissionQuest and Settings.DelvingMissionActive) and
        (data.All_Delving_Tiers == nil or data.All_Delving_Tiers or data.All_Delving_Tiers == false and tostring(data.Delving_Tier) == tostring(Settings.DelvingTier)) and 
        (data.OnlyPvP and Settings.EttenmoorsState or not(data.OnlyPvP)) and
        (data.ClassLoad == "All" or data.ClassLoad == self.ClassLut[self.player:GetClass()]) and 
        (data.RoleLoad == "All" or data.RoleLoad == Roles:GetRoleText()) and 
        (data.LineLoad == "All" or data.LineLoad == Roles:GetLineText()) and
        (data.InstanceLoad == "All" and not(data.AnyInstance) or Settings.InstanceState and data.AnyInstance or data.InstanceLoad == self.DungeonNameTable[Settings.InstanceVersion] and not(data.AnyInstance)) and 
        (not(data.CharLoad) or data.CharacterLoad and data.CharacterLoad[self.player:GetName()]) and
        (data.AllTiers == nil or data.AllTiers or data.AllTiers == false and data.InstanceTier == Settings.InstanceTier)
end

function PrimeUIHandler:RecheckPermissions()
    for key, value in pairs(self.UIData) do
        local load = self:Permission(value)
        if load then
            -- Load if not loaded
            if not(self.tools[value.Name]) then
                self.tools[value.Name] = PrimeUIObjects.PrimeUI(value.PrimeUI, value.Name, value.X, value.Y, value.AdvancedMode, value.OOCLoad, value.ICLoad, value.Anchor_Advanced_X, value.Anchor_Advanced_Y)
                self.tools[value.Name]:ReloadWithDelay()
            else
                self.tools[value.Name]:ReloadWithDelay()
            end
        else
            -- Unload if loaded
            if self.tools[value.Name] then
                self.tools[value.Name]:UnloadObject()
                self.tools[value.Name]:DeleteObject()
                self.tools[value.Name] = nil
            end
        end
    end
end

function PrimeUIHandler:LoadUI()
    for key, value in pairs(self.tools) do       
        value:Setup()
    end
end

function PrimeUIHandler:AddUI(data)
    if not(self.tools[data.Name]) then
        table.insert(self.UIData, data)
        self.tools[data.Name] = PrimeUIObjects.PrimeUI(data.PrimeUI, data.Name, data.X, data.Y, data.AdvancedMode, data.OOCLoad, data.ICLoad, data.Anchor_Advanced_X, data.Anchor_Advanced_Y)
        return true
    else
        ChatOutput("Failed to add PrimeUI: Rejected by handler")
        return false
    end
end

function PrimeUIHandler:UnloadUI()
    for key, value in pairs(self.tools) do       
        value:UnloadObject()
    end
end

function PrimeUIHandler:GetData()
    return self.UIData
end

function PrimeUIHandler:Refresh(name, newData)
    if self.tools[name] then
        self.tools[name]:UnloadObject()
        self.tools[name]:Setup(newData)
    end
end

function PrimeUIHandler:RefreshAndReorder(name, newData)
    if self.tools[name] then
        self.tools[name]:UnloadObject()
        self.tools[name]:Setup(newData)
    end
end

function PrimeUIHandler:Move(name)
    if self.tools[name] then
        if self.tools[name].groupMode then
            local table = self.tools[name]:ToggleAdvancedMove()
            for key, value in pairs(self.UIData) do
                if value.Name == name then
                    self.UIData[key].PrimeUI = table
                end
            end
        else
            local X, Y = self.tools[name]:ToggleMove()
            for key, value in pairs(self.UIData) do
                if value.Name == name then
                    self.UIData[key].X = X
                    self.UIData[key].Y = Y
                end
            end
        end
    end
end

function PrimeUIHandler:ChangeUIMetaData(newData)
    for key, value in pairs(self.UIData) do
        if newData.Name == value.Name then
            self.UIData[key] = newData
            if self:Permission(newData) then
                if self.tools[newData.Name] then
                    self:Refresh(newData.Name, self.UIData[key]["PrimeUI"])
                end
            elseif self.tools[newData.Name] then
                self.tools[newData.Name]:UnloadObject()
            end
            self.UIData[key] = newData
            break
        end
    end

    self:RecheckPermissions()
end

function PrimeUIHandler:Delete(name)
    for key, value in pairs(self.UIData) do
        if value.Name == name then
            if self.tools[name] then
                self.tools[name]:UnloadObject()
                self.tools[name]:DeleteObject()
                self.tools[name] = nil
            end
            self.UIData[key] = nil
        end
    end
end

function PrimeUIHandler:MoveAll()
    for key, value in pairs(self.UIData) do
        if self.tools[value.Name] then
            if self.tools[value.Name].groupMode then
                self.UIData[key].PrimeUI = self.tools[value.Name]:ToggleAdvancedMove()
            else
                local X, Y = self.tools[value.Name]:ToggleMove()
                self.UIData[key].X = X
                self.UIData[key].Y = Y
            end 
        end
    end
end

function PrimeUIHandler:Save()
    SaveTable(Turbine.DataScope.Account, "PrimeUI", self.UIData)
end

function PrimeUIHandler:ReloadWithDelay()
end

function PrimeUIHandler:HideWindow(value)
    for key, value in pairs(self.tools) do
        value:SetVisible(value)
    end
end

function PrimeUIHandler:GetPrimeUICount()
    return tablelength(self.tools)
end


function PrimeUIHandler:GetTotalPrimeUICount()
    return tablelength(self.UIData)
end